-- $Id: test_one.adb,v 1.5 2008/11/23 01:54:28 baker Exp $

--  This is a first test program for simulation packages.
--  It only tests one task, with one policy.
--  Anything more complicated would be more difficult to debug.

with Ada.Exceptions;
with Ada.Text_IO; use Ada.Text_IO;
with Ada.Integer_Text_IO; use Ada.Integer_Text_IO;
with Ada.Float_Text_IO; use Ada.Float_Text_IO;
with Error_Log; use Error_Log;
with Tasks;
with Simulator;
with Threads;
with Threads.Sched_DSS;
with Workload_Models;
with Aperiodic_Server_Parameters; use Aperiodic_Server_Parameters;
with Aperiodic_Workloads;
with Random_Tools;
with Stats;

procedure Test_One is

   Aperiodic_Tasks : array (1 .. 1) of aliased Tasks.Task_Object;

   Aperiodic_Loads : array (1 .. 1) of aliased Aperiodic_Workloads.Object;

   Aperiodic_Policies : array (Aperiodic_Tasks'Range)
     of aliased Threads.Sched_DSS.Object;

   Aperiodic_Load_Parameters : array (Aperiodic_Tasks'range)
     of aliased Aperiodic_Workloads.Parameters :=
     (1 => (Mean_Interarrival_Time => 3.0, Mean_Execution_Time => 2.0));

   Aperiodic_Policy_Parameters : array (Aperiodic_Tasks'range)
     of Aperiodic_Server_Parameters.Parameters :=
     (1 => (Budget => 1, Budget_Interval => 2));

   Test_Name : constant String := "test_one";

begin

   Error_Log.Open (Test_Name);
   Error_Log.Debug_Level := 10;

   --  run one simulation
   --  just one task, for now

   --  set up one workload model object and one
   --  scheduling policy object per aperiodic task

   for K in Aperiodic_Tasks'Range loop
      Aperiodic_Loads (K).Bind_Parms (Aperiodic_Load_Parameters (K));
      Aperiodic_Policies (K).Bind_Parms (Aperiodic_Policy_Parameters (K));
      Tasks.Bind
        (Aperiodic_Tasks (K)'Unchecked_Access,
         M => Aperiodic_Loads (K)'Unchecked_Access,
         P => Aperiodic_Policies (K)'Unchecked_Access,
         Name => Integer'Image (K));
   end loop;

   --  start the pseudo-random number generator that
   --  determines the "random" events in the simulation

   Random_Tools.Start (3, 5, 7, 11, False);

   --  initialize/reset the tasks,
   --  and thereby everything that depends on them

   Tasks.Initialize;
   Simulator.Run (Normal_Stop_Time => 10,
                  Latest_Stop_Time => 20);

   for K in Aperiodic_Tasks'Range loop
      declare
         S : Stats.Data := Tasks.Get_Stats (Aperiodic_Tasks (K)'Unchecked_Access);
      begin
         --  just the basics, for now
         Put ("Max_Response_Time = ");
         Put (Integer (S.Max_Response_Time));
         New_Line;
         Put ("Total_Response_Time = ");
         Put (Integer (S.Total_Response_Time));
         New_Line;
         Put ("Job_Count = ");
         Put (Integer (S.Job_Count));
         New_Line;
         Put ("Missed_Deadlines = ");
         Put (Integer (S.Missed_Deadlines));
         New_Line;
      end;
   end loop;

   Error_Log.Close;

exception
   when E : others =>
      Put (Log, "exception " & Ada.Exceptions.Exception_Name (E));
      Put (Log, " in " & Test_Name & " at ");
      Put_Line (Log, Ada.Exceptions.Exception_Message (E));
      Error_Log.Close;
      raise;
end Test_one;
